@@ -3,7 +3,6 @@
 import base64
 import collections
 import hashlib
-import json
 import pipes
 import re
 import zlib
@@ -11,9 +10,9 @@ import zlib
 import colorama
 from packaging.requirements import InvalidRequirement
 from packaging.version import Version, InvalidVersion
+import ujson
 
 import wiz.exception
-import wiz.mapping
 import wiz.symbol
 from ._requirement import Requirement
 
@@ -24,15 +23,14 @@ _INFINITY_VERSION = 9999
 def get_requirement(content):
     """Return the corresponding requirement instance from *content*.
 
-    The requirement returned is a :class:`packaging.requirements.Requirement`
-    instance.
+    :param content: String representing a requirement, with or without
+        version specifier or variant (e.g. "maya", "nuke >= 10, < 11",
+        "ldpk-nuke[10.0]").
 
-    *content* must be a string which represent a requirement, with or without
-    version specifier or variant (e.g. "maya", "nuke >= 10, < 11",
-    "ldpk-nuke[10.0]").
+    :return: Instance of :class:`packaging.requirements.Requirement`.
 
-    Raises :exc:`wiz.exception.InvalidRequirement` if the requirement is
-    incorrect.
+    :raise: :exc:`wiz.exception.InvalidRequirement` if the requirement is
+        incorrect.
 
     """
     try:
@@ -46,11 +44,11 @@ def get_requirement(content):
 def get_version(content):
     """Return the corresponding version instance from *content*.
 
-    The version returned is a :class:`packaging.version.Version` instance.
+    :param content: String representing a version (e.g. "2018", "0.1.0").
 
-    *content* must be a string which represent a version (e.g. "2018", "0.1.0").
+    :return: Instance of :class:`packaging.version.Version`.
 
-    Raises :exc:`wiz.exception.InvalidVersion` if the version is incorrect.
+    :raise: :exc:`wiz.exception.InvalidVersion` if the version is incorrect.
 
     """
     try:
@@ -75,11 +73,16 @@ def is_overlapping(requirement1, requirement2):
         >>> is_overlapping(Requirement("foo >= 10"), Requirement("foo < 8"))
         False
 
-    *requirement1* and *requirement2* should be instances of
-    :class:`packaging.requirements.Requirement`.
+    :param requirement1: Instance of
+        :class:`packaging.requirements.Requirement`.
 
-    Raise :exc:`wiz.exception.GraphResolutionError` if requirements cannot
-    be compared.
+    :param requirement2: Instance of
+        :class:`packaging.requirements.Requirement`.
+
+    :return: Boolean value.
+
+    :raise: :exc:`wiz.exception.GraphResolutionError` if requirements cannot
+        be compared.
 
     """
     if requirement1.name != requirement2.name:
@@ -91,16 +94,14 @@ def is_overlapping(requirement1, requirement2):
     r1 = extract_version_ranges(requirement1)
     r2 = extract_version_ranges(requirement2)
     return (
-        (r2[-1][1] is None or r2[-1][1] >= r1[0][0]) and
-        (r1[-1][1] is None or r1[-1][1] >= r2[0][0])
+        (r2[-1][1] is None or r1[0][0] is None or r2[-1][1] >= r1[0][0]) and
+        (r1[-1][1] is None or r2[0][0] is None or r1[-1][1] >= r2[0][0])
     )
 
 
 def extract_version_ranges(requirement):
     """Extract version ranges from *requirement*.
 
-    A list of version tuples is returned
-
     Requirement could contain the following specifiers:
 
     * `Compatible release
@@ -131,20 +132,24 @@ def extract_version_ranges(requirement):
         >>> extract_version_ranges(Requirement("foo >= 2, < 3"))
         [((2,), (2, 9999))]
 
-    *requirement* should be an instance of
-    :class:`packaging.requirements.Requirement`.
+    :param requirement: Instance of :class:`packaging.requirements.Requirement`.
 
-    :exc:`wiz.exception.InvalidVersion` is raised if the version extracted from
-    the specifier is incorrect.
+    :return: List of version tuples.
 
-    :exc:`wiz.exception.InvalidRequirement` is raised if the specifier operator
-    is not accepted or if the requirement does not allow any versions to be
-    reached.
+    :raise: :exc:`wiz.exception.InvalidVersion` if the version extracted from
+        the specifier is incorrect.
+
+    :raise: :exc:`wiz.exception.InvalidRequirement` if the specifier operator
+        is not accepted or if the requirement does not allow any versions to be
+        reached.
 
     """
     version_ranges = [(None, None)]
 
-    for specifier in requirement.specifier:
+    # Sort specifiers for deterministic results.
+    for specifier in sorted(
+        requirement.specifier, key=lambda s: (s.version, s.operator)
+    ):
         # Extract version number.
         _version = wiz.utility.get_version(
             re.sub(r"\.\*$", "", specifier.version)
@@ -202,31 +207,37 @@ def extract_version_ranges(requirement):
 def _update_maximum_version(version, ranges):
     """Update version *ranges* with maximum *version*.
 
-    *version* should be a version release tuple (e.g. (1,2,3)).
-
-    *ranges* should be an ordered list of tuples containing two
-    ordered version release tuples (e.g [((1,2,3), (1,3,0)), (1,3,3), (1,4))]).
-
     Example::
 
-        >>> ranges = [((1,2,3), (1,3,0)), ((1,3,3), (1,4))]
+        >>> ranges = [((1, 2, 3), (1, 3, 0)), ((1, 3, 3), (1, 4))]
         >>> _update_maximum_version((1, 2, 3), ranges)
         >>> print(ranges)
         [((1, 2, 3), (1, 2, 3))]
 
+    :param version: Tuple representing a version (e.g. (1,2,3)).
+
+    :param ranges: ordered list of tuples containing two ordered version release
+        tuples (e.g [((1,2,3), (1,3,0)), (1,3,3), (1,4))]).
+
+    .. warning::
+
+        The *ranges* list will be mutated.
+
     """
     _ranges = []
 
     if ranges[0][0] is not None and version < ranges[0][0]:
         raise wiz.exception.InvalidRequirement(
-            "The requirement is incorrect as maximum value '{}' cannot be set"
+            "The requirement is incorrect as maximum value '{}' cannot be set "
             "when minimum value is '{}'.".format(
                 ".".join(str(v) for v in version),
-                ".".join(str(v) for v in ranges[-1][1])
+                ".".join(str(v) for v in ranges[0][0])
             )
         )
 
     for start_version, end_version in ranges:
+        if start_version and start_version > version:
+            break
         if not end_version or end_version >= version:
             _ranges.append((start_version, version))
             break
@@ -239,24 +250,28 @@ def _update_maximum_version(version, ranges):
 def _update_minimum_version(version, ranges):
     """Update version *ranges* with minimum *version*.
 
-    *version* should be a version release tuple (e.g. (1,2,3)).
-
-    *ranges* should be an ordered list of tuples containing two
-    ordered version release tuples (e.g [((1,2,3), (1,3,0)), (1,3,3), (1,4))).
-
     Example::
 
-        >>> ranges = [((1,2,3), (1,3,0)), ((1,3,3), (1,4))]
+        >>> ranges = [((1, 2, 3), (1, 3, 0)), ((1, 3, 3), (1, 4))]
         >>> _update_minimum_version((1, 3, 3), ranges)
         >>> print(ranges)
         [((1, 3, 3), (1, 4))]
 
+    :param version: Tuple representing a version (e.g. (1,2,3)).
+
+    :param ranges: ordered list of tuples containing two ordered version release
+        tuples (e.g [((1,2,3), (1,3,0)), (1,3,3), (1,4))]).
+
+    .. warning::
+
+        The *ranges* list will be mutated.
+
     """
     _ranges = []
 
     if ranges[-1][1] is not None and version > ranges[-1][1]:
         raise wiz.exception.InvalidRequirement(
-            "The requirement is incorrect as minimum value '{}' cannot be set"
+            "The requirement is incorrect as minimum value '{}' cannot be set "
             "when maximum value is '{}'.".format(
                 ".".join(str(v) for v in version),
                 ".".join(str(v) for v in ranges[-1][1])
@@ -264,6 +279,8 @@ def _update_minimum_version(version, ranges):
         )
 
     for start_version, end_version in reversed(ranges):
+        if end_version and end_version < version:
+            break
         if not start_version or start_version <= version:
             _ranges = [(version, end_version)] + _ranges
             break
@@ -274,14 +291,7 @@ def _update_minimum_version(version, ranges):
 
 
 def _update_version_ranges(excluded, ranges):
-    """Update version *ranges* from excluded *version_range*.
-
-    *excluded* should be a tuple containing two ordered version release
-    tuples (e.g. ((1,2,3), (1,3,0))). These two versions are included in the
-    *ranges*, but all versions in between should be excluded.
-
-    *ranges* should be an ordered list of tuples containing two
-    ordered version release tuples (e.g [((1,2,3), (1,3,0)), (1,3,3), (1,4))).
+    """Update version *ranges* from *excluded* version range.
 
     Example::
 
@@ -290,12 +300,23 @@ def _update_version_ranges(excluded, ranges):
         >>> print(ranges)
         [((1, 2, 3), (1, 2, 3)), ((1, 3, 3), (1, 4))]
 
+    :param excluded: Tuple containing two ordered version release tuples (e.g.
+        ((1,2,3), (1,3,0))). These two versions are included in *ranges*, but
+        all versions in between should be excluded.
+
+    :param ranges: ordered list of tuples containing two ordered version release
+        tuples (e.g [((1,2,3), (1,3,0)), (1,3,3), (1,4))]).
+
+    .. warning::
+
+        The *ranges* list will be mutated.
+
     """
     _ranges = []
 
     for r in ranges:
-        out_before = r[0] is not None and r[0] > excluded[1]
-        out_after = r[1] is not None and r[1] < excluded[0]
+        out_before = r[0] is not None and r[0] >= excluded[1]
+        out_after = r[1] is not None and r[1] <= excluded[0]
 
         # Exclusion zone is outside of range.
         if out_before or out_after:
@@ -340,14 +361,6 @@ def _increment_version(version, delta=1, add_subversion=True):
 
     This will attempt to increase to the nearest possible version tuple.
 
-    *version* should be a version release tuple (e.g. (1, 2, 3)).
-
-    *delta* should be the number to add to the minimal the *version*. Default
-    is 1.
-
-    *add_subversion* indicates whether a sub-version should be used instead
-    of only increasing the minimal release version. Default is True.
-
     Example::
 
         >>> _increment_version((1, 2, 0))
@@ -359,6 +372,15 @@ def _increment_version(version, delta=1, add_subversion=True):
         >>> _increment_version((1, 1, 1), delta=3)
         (1, 1, 1, 3)
 
+    :param version: Tuple representing a version (e.g. (1,2,3)).
+
+    :param delta: Number to add to the minimal the *version*. Default is 1.
+
+    :param add_subversion: Indicate whether a sub-version should be used instead
+        of only increasing the minimal release version. Default is True.
+
+    :return: New version tuple.
+
     """
     if not add_subversion:
         return version[:-1] + (version[-1] + delta,)
@@ -370,8 +392,6 @@ def _decrement_version(version):
 
     This will attempt to decrease to the nearest possible version tuple.
 
-    *version* should be a version release tuple (e.g. (1, 2, 3)).
-
     Example::
 
         >>> _decrement_version((1,))
@@ -386,6 +406,10 @@ def _decrement_version(version):
         >>> _decrement_version((1, 1, 1))
         (1, 1, 0, 9999)
 
+    :param version: Tuple representing a version (e.g. (1,2,3)).
+
+    :return: New version tuple.
+
     """
     index = -1
     while version[index] == 0:
@@ -399,10 +423,12 @@ def encode(element):
 
     *element* is serialized first, then encoded into :term:`base64`.
 
-    Raises :exc:`TypeError` if *element* is not JSON serializable.
+    :param element: Content to encode.
+
+    :raise: :exc:`TypeError` if *element* is not JSON serializable.
 
     """
-    return base64.b64encode(zlib.compress(json.dumps(element)))
+    return base64.b64encode(zlib.compress(ujson.dumps(element).encode("utf-8")))
 
 
 def decode(element):
@@ -410,10 +436,12 @@ def decode(element):
 
     *element* is decoded first from :term:`base64`, then deserialized.
 
-    Raises :exc:`TypeError` if *element* cannot be decoded or deserialized..
+    :param element: Content to decode.
+
+    :raise: :exc:`TypeError` if *element* cannot be decoded or deserialized.
 
     """
-    return json.loads(zlib.decompress(base64.b64decode(element)))
+    return ujson.loads(zlib.decompress(base64.b64decode(element)))
 
 
 def compute_label(definition):
@@ -426,15 +454,17 @@ def compute_label(definition):
         "'baz' [0.2.0] (linux : el =! 7)"
         "'bim' (linux : el >= 6, < 7)"
 
-    *definition* should be a :class:`wiz.definition.Definition` instance.
+    :param definition: Instance of :class:`wiz.definition.Definition`.
+
+    :return: String representing definition.
 
     """
     label = "'{}'".format(definition.qualified_identifier)
 
-    if definition.get("version"):
+    if definition.version:
         label += " [{}]".format(definition.version)
 
-    if definition.get("system"):
+    if definition.system:
         system_identifier = compute_system_label(definition)
         label += " ({})".format(system_identifier)
 
@@ -446,12 +476,15 @@ def compute_system_label(definition):
 
     The system identifier should be in the form of::
 
+        "noarch"
         "linux : x86_64 : el >= 7, < 8"
         "centos >= 7, < 8"
         "x86_64 : el >= 7, < 8"
         "windows"
 
-    *definition* should be a :class:`wiz.definition.Definition` instance.
+    :param definition: Instance of :class:`wiz.definition.Definition`.
+
+    :return: String representing system identifier.
 
     """
     elements = [
@@ -468,23 +501,28 @@ def compute_file_name(definition):
     The file name should be in the form of::
 
         "foo.json"
+        "namespace-foo-0.1.0.json"
         "foo-0.1.0.json"
         "foo-0.1.0-M2Uq9Esezm-m00VeWkTzkQIu3T4.json"
 
-    *definition* should be a :class:`wiz.definition.Definition` instance.
+    :param definition: Instance of :class:`wiz.definition.Definition`.
+
+    :return: File name representing definition.
 
     """
     name = definition.identifier
 
-    if definition.get("version"):
+    if definition.namespace:
+        name = "{}-{}".format(definition.namespace, name)
+
+    if definition.version:
         name += "-{}".format(definition.version)
 
-    if definition.get("system"):
+    if definition.system:
         system_identifier = wiz.utility.compute_system_label(definition)
-        encoded = base64.urlsafe_b64encode(
-            hashlib.sha1(re.sub(r"(\s+|:+)", "", system_identifier)).digest()
-        )
-        name += "-{}".format(encoded.rstrip("="))
+        data = re.sub(r"(\s+|:+)", "", system_identifier).encode("utf-8")
+        encoded = base64.urlsafe_b64encode(hashlib.sha1(data).digest())
+        name += "-{}".format(encoded.rstrip(b"=").decode("utf-8"))
 
     return "{}.json".format(name)
 
@@ -500,6 +538,10 @@ def combine_command(elements):
 
         python2.7 -c 'import os; print(os.environ["HOME"])'
 
+    :param elements: List of strings constituting the command line to execute
+        (e.g. ["app_exe", "--option", "value"])
+
+
     """
     return " ".join([pipes.quote(element) for element in elements])
 
@@ -510,6 +552,12 @@ def colored_text(message, color):
     Available color names are: "black", "red", "green", "yellow", "blue",
     "magenta", "cyan" and "white".
 
+    :param message: String to colorize.
+
+    :param color: Symbol of color to apply to *message*.
+
+    :return: Colorized message.
+
     """
     return (
         getattr(colorama.Fore, color.upper()) + message +
