@@ -1,14 +1,15 @@
 # :coding: utf-8
 
 import pytest
-import mock
 import types
-import itertools
 import re
 
+import six.moves
+
 from wiz.utility import Requirement, Version
 import wiz.graph
 import wiz.package
+import wiz.definition
 import wiz.exception
 
 
@@ -313,7 +314,7 @@ def test_generate_variant_combinations(
     mocked_graph.node = lambda _id: mocker.Mock(
         identifier=_id,
         package=mocker.Mock(
-            variant_name=re.search("(?<=\[).+(?=\])", _id).group(0)
+            variant_identifier=re.search(r"(?<=\[).+(?=\])", _id).group(0)
         )
     )
 
@@ -321,7 +322,7 @@ def test_generate_variant_combinations(
         mocked_graph, variant_groups
     )
     assert isinstance(results, types.GeneratorType) is True
-    for combination, _expected in itertools.izip_longest(results, expected):
+    for combination, _expected in six.moves.zip_longest(results, expected):
         assert combination[0] == mocked_graph
         assert combination[1] == _expected
 
@@ -979,7 +980,9 @@ def test_extract_ordered_packages(
 ):
     """Extract ordered packages from graph."""
     package_mapping = {
-        _id: wiz.package.Package({"identifier": _id}) for _id in identifiers
+        _id: wiz.package.Package(
+            wiz.definition.Definition({"identifier": _id})
+        ) for _id in identifiers
     }
 
     nodes = [
@@ -1051,28 +1054,28 @@ def test_graph_find_matching_identifiers(
             definition=mocker.Mock(qualified_identifier="A"),
             package=mocker.Mock(
                 version=Version("0.1.0"),
-                qualified_identifier="A==0.1.0"
+                identifier="A==0.1.0"
             )
         ),
         "A==2.4.5": mocker.Mock(
             definition=mocker.Mock(qualified_identifier="A"),
             package=mocker.Mock(
                 version=Version("2.4.5"),
-                qualified_identifier="A==2.4.5"
+                identifier="A==2.4.5"
             )
         ),
         "Name1::B": mocker.Mock(
             definition=mocker.Mock(qualified_identifier="Name1::B"),
             package=mocker.Mock(
                 version=None,
-                qualified_identifier="Name1::B"
+                identifier="Name1::B"
             )
         ),
         "B==1": mocker.Mock(
             definition=mocker.Mock(qualified_identifier="B"),
             package=mocker.Mock(
                 version=Version("1"),
-                qualified_identifier="B==1"
+                identifier="B==1"
             )
         ),
     }
@@ -1091,56 +1094,56 @@ def test_graph_variant_groups(mocker):
         "A[V1]==0.1.0": mocker.Mock(
             identifier="A[V1]==0.1.0",
             package=mocker.Mock(
-                variant_name="V1",
+                variant_identifier="V1",
                 version=Version("0.1.0")
             )
         ),
         "A[V2]==0.1.0": mocker.Mock(
             identifier="A[V2]==0.1.0",
             package=mocker.Mock(
-                variant_name="V2",
+                variant_identifier="V2",
                 version=Version("0.1.0")
             )
         ),
         "B[V1]==0.1.0": mocker.Mock(
             identifier="B[V1]==0.1.0",
             package=mocker.Mock(
-                variant_name="V1",
+                variant_identifier="V1",
                 version=Version("0.1.0")
             )
         ),
         "B[V1]==0.2.0": mocker.Mock(
             identifier="B[V1]==0.2.0",
             package=mocker.Mock(
-                variant_name="V1",
+                variant_identifier="V1",
                 version=Version("0.2.0")
             )
         ),
         "B[V2]==0.1.0": mocker.Mock(
             identifier="B[V2]==0.1.0",
             package=mocker.Mock(
-                variant_name="V2",
+                variant_identifier="V2",
                 version=Version("0.1.0")
             )
         ),
         "C[V1]==0.1.0": mocker.Mock(
             identifier="C[V1]==0.1.0",
             package=mocker.Mock(
-                variant_name="V1",
+                variant_identifier="V1",
                 version=Version("0.1.0")
             )
         ),
         "D[V1]==0.1.0": mocker.Mock(
             identifier="D[V1]==0.1.0",
             package=mocker.Mock(
-                variant_name="V1",
+                variant_identifier="V1",
                 version=Version("0.1.0")
             )
         ),
         "D[V1]==0.2.0": mocker.Mock(
             identifier="D[V1]==0.2.0",
             package=mocker.Mock(
-                variant_name="V1",
+                variant_identifier="V1",
                 version=Version("0.2.0")
             )
         )
@@ -1245,14 +1248,18 @@ def test_graph_update_from_requirements(
     graph = wiz.graph.Graph(mocked_resolver)
 
     _mapping = {
-        "A==0.2.0": wiz.package.Package({
-            "identifier": "A==0.2.0",
-            "definition": mocker.Mock(qualified_identifier="A")
-        }),
-        "B==2.1.1": wiz.package.Package({
-            "identifier": "B==2.1.1",
-            "definition": mocker.Mock(qualified_identifier="B")
-        }),
+        "A==0.2.0": wiz.package.Package(
+            wiz.definition.Definition({
+                "identifier": "A",
+                "version": "0.2.0"
+            })
+        ),
+        "B==2.1.1": wiz.package.Package(
+            wiz.definition.Definition({
+                "identifier": "B",
+                "version": "2.1.1"
+            })
+        ),
     }
 
     mocked_package_extract.side_effect = [
@@ -1264,20 +1271,20 @@ def test_graph_update_from_requirements(
         [Requirement("A"), Requirement("B>=2")], graph.ROOT
     )
 
-    assert graph.to_dict() == {
-        "identifier": mock.ANY,
+    assert graph.data() == {
+        "identifier": mocker.ANY,
         "node_mapping": {
             "A==0.2.0": {
                 "package": {
                     "identifier": "A==0.2.0",
-                    "definition": mocker.ANY
+                    "version": "0.2.0"
                 },
                 "parents": ["root"]
             },
             "B==2.1.1": {
                 "package": {
                     "identifier": "B==2.1.1",
-                    "definition": mocker.ANY
+                    "version": "2.1.1"
                 },
                 "parents": ["root"]
             }
@@ -1306,29 +1313,39 @@ def test_graph_update_from_requirements_with_dependencies(
     graph = wiz.graph.Graph(mocked_resolver)
 
     _mapping = {
-        "A==0.1.0": wiz.package.Package({
-            "identifier": "A==0.1.0",
-            "definition": mocker.Mock(qualified_identifier="A"),
-            "requirements": [Requirement("B>=2"), Requirement("C")],
-        }),
-        "B==3.0.0": wiz.package.Package({
-            "identifier": "B==3.0.0",
-            "definition": mocker.Mock(qualified_identifier="B")
-        }),
-        "C==1.2.3": wiz.package.Package({
-            "identifier": "C==1.2.3",
-            "definition": mocker.Mock(qualified_identifier="C"),
-            "requirements": [Requirement("D")],
-        }),
-        "D==0.1.0": wiz.package.Package({
-            "identifier": "D==0.1.0",
-            "definition": mocker.Mock(qualified_identifier="D"),
-            "requirements": [Requirement("E")],
-        }),
-        "E==0.2.0": wiz.package.Package({
-            "identifier": "E==0.2.0",
-            "definition": mocker.Mock(qualified_identifier="E")
-        }),
+        "A==0.1.0": wiz.package.Package(
+            wiz.definition.Definition({
+                "identifier": "A",
+                "version": "0.1.0",
+                "requirements": ["B>=2", "C"],
+            })
+        ),
+        "B==3.0.0": wiz.package.Package(
+            wiz.definition.Definition({
+                "identifier": "B",
+                "version": "3.0.0",
+            })
+        ),
+        "C==1.2.3": wiz.package.Package(
+            wiz.definition.Definition({
+                "identifier": "C",
+                "version": "1.2.3",
+                "requirements": ["D"],
+            })
+        ),
+        "D==0.1.0": wiz.package.Package(
+            wiz.definition.Definition({
+                "identifier": "D",
+                "version": "0.1.0",
+                "requirements": ["E"],
+            })
+        ),
+        "E==0.2.0": wiz.package.Package(
+            wiz.definition.Definition({
+                "identifier": "E",
+                "version": "0.2.0",
+            })
+        ),
     }
 
     mocked_package_extract.side_effect = [
@@ -1341,28 +1358,28 @@ def test_graph_update_from_requirements_with_dependencies(
 
     graph.update_from_requirements([Requirement("A")], graph.ROOT)
 
-    assert graph.to_dict() == {
-        "identifier": mock.ANY,
+    assert graph.data() == {
+        "identifier": mocker.ANY,
         "node_mapping": {
             "A==0.1.0": {
                 "package": {
                     "identifier": "A==0.1.0",
-                    "definition": mocker.ANY,
-                    "requirements": ["B >=2", "C"]
+                    "version": "0.1.0",
+                    "requirements": ["B>=2", "C"]
                 },
                 "parents": ["root"]
             },
             "B==3.0.0": {
                 "package": {
                     "identifier": "B==3.0.0",
-                    "definition": mocker.ANY,
+                    "version": "3.0.0",
                 },
                 "parents": ["A==0.1.0"]
             },
             "C==1.2.3": {
                 "package": {
                     "identifier": "C==1.2.3",
-                    "definition": mocker.ANY,
+                    "version": "1.2.3",
                     "requirements": ["D"]
                 },
                 "parents": ["A==0.1.0"]
@@ -1370,7 +1387,7 @@ def test_graph_update_from_requirements_with_dependencies(
             "D==0.1.0": {
                 "package": {
                     "identifier": "D==0.1.0",
-                    "definition": mocker.ANY,
+                    "version": "0.1.0",
                     "requirements": ["E"]
                 },
                 "parents": ["C==1.2.3"]
@@ -1378,7 +1395,7 @@ def test_graph_update_from_requirements_with_dependencies(
             "E==0.2.0": {
                 "package": {
                     "identifier": "E==0.2.0",
-                    "definition": mocker.ANY,
+                    "version": "0.2.0",
                 },
                 "parents": ["D==0.1.0"]
             }
@@ -1418,22 +1435,20 @@ def test_graph_update_from_requirements_with_variants(
     """Update graph from requirements with variants of definition."""
     graph = wiz.graph.Graph(mocked_resolver)
 
+    definition = wiz.definition.Definition({
+        "identifier": "A",
+        "version": "0.2.0",
+        "variants": [
+            {"identifier": "V1"},
+            {"identifier": "V2"},
+            {"identifier": "V3"},
+        ]
+    })
+
     _mapping = {
-        "A[V1]==0.2.0": wiz.package.Package({
-            "identifier": "A[V1]==0.2.0",
-            "variant-name": "V1",
-            "definition": mocker.Mock(qualified_identifier="A")
-        }),
-        "A[V2]==0.2.0": wiz.package.Package({
-            "identifier": "A[V2]==0.2.0",
-            "variant-name": "V2",
-            "definition": mocker.Mock(qualified_identifier="A")
-        }),
-        "A[V3]==0.2.0": wiz.package.Package({
-            "identifier": "A[V3]==0.2.0",
-            "variant-name": "V3",
-            "definition": mocker.Mock(qualified_identifier="A")
-        }),
+        "A[V1]==0.2.0": wiz.package.Package(definition, variant_index=0),
+        "A[V2]==0.2.0": wiz.package.Package(definition, variant_index=1),
+        "A[V3]==0.2.0": wiz.package.Package(definition, variant_index=2),
     }
 
     mocked_package_extract.side_effect = [
@@ -1446,30 +1461,30 @@ def test_graph_update_from_requirements_with_variants(
 
     graph.update_from_requirements([Requirement("A")], graph.ROOT)
 
-    assert graph.to_dict() == {
-        "identifier": mock.ANY,
+    assert graph.data() == {
+        "identifier": mocker.ANY,
         "node_mapping": {
             "A[V1]==0.2.0": {
                 "package": {
                     "identifier": "A[V1]==0.2.0",
-                    "variant-name": "V1",
-                    "definition": mocker.ANY,
+                    "version": "0.2.0",
+                    "variant-identifier": "V1",
                 },
                 "parents": ["root"]
             },
             "A[V2]==0.2.0": {
                 "package": {
                     "identifier": "A[V2]==0.2.0",
-                    "variant-name": "V2",
-                    "definition": mocker.ANY,
+                    "version": "0.2.0",
+                    "variant-identifier": "V2",
                 },
                 "parents": ["root"]
             },
             "A[V3]==0.2.0": {
                 "package": {
                     "identifier": "A[V3]==0.2.0",
-                    "variant-name": "V3",
-                    "definition": mocker.ANY,
+                    "version": "0.2.0",
+                    "variant-identifier": "V3",
                 },
                 "parents": ["root"]
             },
@@ -1507,16 +1522,20 @@ def test_graph_update_from_requirements_with_namespaces(
     graph = wiz.graph.Graph(mocked_resolver)
 
     _mapping = {
-        "Foo::A==0.2.0": wiz.package.Package({
-            "identifier": "A==0.2.0",
-            "namespace": "Foo",
-            "definition": mocker.Mock(qualified_identifier="A")
-        }),
-        "Foo::B==2.1.1": wiz.package.Package({
-            "identifier": "B==2.1.1",
-            "namespace": "Foo",
-            "definition": mocker.Mock(qualified_identifier="B")
-        })
+        "Foo::A==0.2.0": wiz.package.Package(
+            wiz.definition.Definition({
+                "identifier": "A",
+                "version": "0.2.0",
+                "namespace": "Foo",
+            })
+        ),
+        "Foo::B==2.1.1": wiz.package.Package(
+            wiz.definition.Definition({
+                "identifier": "B",
+                "version": "2.1.1",
+                "namespace": "Foo",
+            })
+        )
     }
 
     mocked_package_extract.side_effect = [
@@ -1528,22 +1547,22 @@ def test_graph_update_from_requirements_with_namespaces(
         [Requirement("A"), Requirement("B>=2")], graph.ROOT
     )
 
-    assert graph.to_dict() == {
-        "identifier": mock.ANY,
+    assert graph.data() == {
+        "identifier": mocker.ANY,
         "node_mapping": {
             "Foo::A==0.2.0": {
                 "package": {
-                    "identifier": "A==0.2.0",
+                    "identifier": "Foo::A==0.2.0",
                     "namespace": "Foo",
-                    "definition": mocker.ANY
+                    "version": "0.2.0"
                 },
                 "parents": ["root"]
             },
             "Foo::B==2.1.1": {
                 "package": {
-                    "identifier": "B==2.1.1",
+                    "identifier": "Foo::B==2.1.1",
                     "namespace": "Foo",
-                    "definition": mocker.ANY
+                    "version": "2.1.1"
                 },
                 "parents": ["root"]
             }
@@ -1559,8 +1578,8 @@ def test_graph_update_from_requirements_with_namespaces(
             }
         },
         "identifiers_per_definition": {
-            "A": ["Foo::A==0.2.0"],
-            "B": ["Foo::B==2.1.1"]
+            "Foo::A": ["Foo::A==0.2.0"],
+            "Foo::B": ["Foo::B==2.1.1"]
         },
         "variants_per_definition": {},
         "conditioned_nodes": [],
@@ -1576,24 +1595,32 @@ def test_graph_update_from_requirements_with_skipped_conditional_packages(
     graph = wiz.graph.Graph(mocked_resolver)
 
     _mapping = {
-        "A==0.2.0": wiz.package.Package({
-            "identifier": "A==0.2.0",
-            "definition": mocker.Mock(qualified_identifier="A")
-        }),
-        "B==2.1.1": wiz.package.Package({
-            "identifier": "B==2.1.1",
-            "definition": mocker.Mock(qualified_identifier="B"),
-            "conditions": [Requirement("C > 2")]
-        }),
-        "C==2.0.4": wiz.package.Package({
-            "identifier": "C==2.0.4",
-            "definition": mocker.Mock(qualified_identifier="C")
-        }),
-        "D==0.1.0": wiz.package.Package({
-            "identifier": "D==0.1.0",
-            "definition": mocker.Mock(qualified_identifier="D"),
-            "conditions": [Requirement("W")]
-        }),
+        "A==0.2.0": wiz.package.Package(
+            wiz.definition.Definition({
+                "identifier": "A",
+                "version": "0.2.0",
+            })
+        ),
+        "B==2.1.1": wiz.package.Package(
+            wiz.definition.Definition({
+                "identifier": "B",
+                "version": "2.1.1",
+                "conditions": ["C > 2"]
+            })
+        ),
+        "C==2.0.4": wiz.package.Package(
+            wiz.definition.Definition({
+                "identifier": "C",
+                "version": "2.0.4",
+            })
+        ),
+        "D==0.1.0": wiz.package.Package(
+            wiz.definition.Definition({
+                "identifier": "D",
+                "version": "0.1.0",
+                "conditions": ["W"]
+            })
+        ),
     }
 
     mocked_package_extract.side_effect = [
@@ -1609,13 +1636,13 @@ def test_graph_update_from_requirements_with_skipped_conditional_packages(
         [Requirement("A"), Requirement("B>=2"), Requirement("D")], graph.ROOT
     )
 
-    assert graph.to_dict() == {
-        "identifier": mock.ANY,
+    assert graph.data() == {
+        "identifier": mocker.ANY,
         "node_mapping": {
             "A==0.2.0": {
                 "package": {
                     "identifier": "A==0.2.0",
-                    "definition": mocker.ANY,
+                    "version": "0.2.0",
                 },
                 "parents": ["root"],
             }
@@ -1631,13 +1658,21 @@ def test_graph_update_from_requirements_with_skipped_conditional_packages(
         "conditioned_nodes": [
             {
                 "requirement": Requirement("B >=2"),
-                "package": mock.ANY,
+                "package": {
+                    "identifier": "B==2.1.1",
+                    "version": "2.1.1",
+                    "conditions": ["C > 2"]
+                },
                 "parent_identifier": "root",
                 "weight": 2
             },
             {
                 "requirement": Requirement("D"),
-                "package": mock.ANY,
+                "package": {
+                    "identifier": "D==0.1.0",
+                    "version": "0.1.0",
+                    "conditions": ["W"]
+                },
                 "parent_identifier": "root",
                 "weight": 3
             }
@@ -1655,15 +1690,19 @@ def test_graph_update_from_requirements_with_used_conditional_packages(
     graph = wiz.graph.Graph(mocked_resolver)
 
     _mapping = {
-        "A==0.2.0": wiz.package.Package({
-            "identifier": "A==0.2.0",
-            "definition": mocker.Mock(qualified_identifier="A")
-        }),
-        "B==2.1.1": wiz.package.Package({
-            "identifier": "B==2.1.1",
-            "definition": mocker.Mock(qualified_identifier="B"),
-            "conditions": [Requirement("A")],
-        })
+        "A==0.2.0": wiz.package.Package(
+            wiz.definition.Definition({
+                "identifier": "A",
+                "version": "0.2.0"
+            })
+        ),
+        "B==2.1.1": wiz.package.Package(
+            wiz.definition.Definition({
+                "identifier": "B",
+                "version": "2.1.1",
+                "conditions": ["A"],
+            })
+        )
     }
 
     mocked_package_extract.side_effect = [
@@ -1677,22 +1716,21 @@ def test_graph_update_from_requirements_with_used_conditional_packages(
         [Requirement("A"), Requirement("B>=2")], graph.ROOT
     )
 
-    assert graph.to_dict() == {
-        "identifier": mock.ANY,
+    assert graph.data() == {
+        "identifier": mocker.ANY,
         "node_mapping": {
             "A==0.2.0": {
                 "package": {
                     "identifier": "A==0.2.0",
-                    "definition": mocker.ANY
+                    "version": "0.2.0"
                 },
                 "parents": ["root"]
             },
             "B==2.1.1": {
                 "package": {
                     "identifier": "B==2.1.1",
-                    "definition": mocker.ANY,
+                    "version": "2.1.1",
                     "conditions": ["A"],
-                    "conditions-processed": True,
                 },
                 "parents": ["root"]
             }
@@ -1710,7 +1748,11 @@ def test_graph_update_from_requirements_with_used_conditional_packages(
         "conditioned_nodes": [
             {
                 "requirement": Requirement("B >=2"),
-                "package": mock.ANY,
+                "package": {
+                    "identifier": "B==2.1.1",
+                    "version": "2.1.1",
+                    "conditions": ["A"],
+                },
                 "parent_identifier": "root",
                 "weight": 2
             },
@@ -1728,15 +1770,19 @@ def test_graph_update_from_requirements_with_errors(
     graph = wiz.graph.Graph(mocked_resolver)
 
     _mapping = {
-        "A==0.2.0": wiz.package.Package({
-            "identifier": "A==0.2.0",
-            "definition": mocker.Mock(qualified_identifier="A")
-        }),
-        "B==2.1.1": wiz.package.Package({
-            "identifier": "B==2.1.1",
-            "definition": mocker.Mock(qualified_identifier="B"),
-            "requirements": ["incorrect1", "incorrect2"]
-        })
+        "A==0.2.0": wiz.package.Package(
+            wiz.definition.Definition({
+                "identifier": "A",
+                "version": "0.2.0"
+            })
+        ),
+        "B==2.1.1": wiz.package.Package(
+            wiz.definition.Definition({
+                "identifier": "B",
+                "version": "2.1.1",
+                "requirements": ["incorrect1", "incorrect2"]
+            })
+        )
     }
 
     mocked_package_extract.side_effect = [
@@ -1750,20 +1796,20 @@ def test_graph_update_from_requirements_with_errors(
         [Requirement("A"), Requirement("B>=2")], graph.ROOT
     )
 
-    assert graph.to_dict() == {
-        "identifier": mock.ANY,
+    assert graph.data() == {
+        "identifier": mocker.ANY,
         "node_mapping": {
             "A==0.2.0": {
                 "package": {
                     "identifier": "A==0.2.0",
-                    "definition": mocker.ANY
+                    "version": "0.2.0"
                 },
                 "parents": ["root"]
             },
             "B==2.1.1": {
                 "package": {
                     "identifier": "B==2.1.1",
-                    "definition": mocker.ANY,
+                    "version": "2.1.1",
                     "requirements": ["incorrect1", "incorrect2"]
                 },
                 "parents": ["root"]
@@ -1799,7 +1845,12 @@ def test_graph_update_from_requirement_existing(
     mocker, mocked_resolver, mocked_package_extract, mocked_queue, options
 ):
     """Update graph from requirement."""
-    package = wiz.package.Package(identifier="A==0.1.0")
+    package = wiz.package.Package(
+        wiz.definition.Definition({
+            "identifier": "A",
+            "version": "0.1.0"
+        })
+    )
     node = mocker.Mock(identifier="_A==0.1.0")
     requirement = Requirement("A")
 
@@ -1817,9 +1868,7 @@ def test_graph_update_from_requirement_existing(
 
     graph._create_node_from_package.assert_not_called()
     graph.create_link.assert_called_once_with(
-        "_A==0.1.0",
-        graph.ROOT,
-        requirement,
+        "_A==0.1.0", graph.ROOT, requirement,
         weight=options.get("weight", 1)
     )
 
@@ -1839,7 +1888,12 @@ def test_graph_update_from_requirement_non_existing(
     mocker, mocked_resolver, mocked_package_extract, mocked_queue, options
 ):
     """Update graph from requirement."""
-    package = wiz.package.Package(identifier="A==0.1.0")
+    package = wiz.package.Package(
+        wiz.definition.Definition({
+            "identifier": "A",
+            "version": "0.1.0"
+        })
+    )
     node = mocker.Mock(identifier="_A==0.1.0")
     requirement = Requirement("A")
 
@@ -1880,12 +1934,11 @@ def test_graph_update_from_requirement_non_existing_with_requirements(
 ):
     """Update graph from requirement."""
     package = wiz.package.Package(
-        identifier="A==0.1.0",
-        requirements=[
-            Requirement("B"),
-            Requirement("C"),
-            Requirement("D")
-        ],
+        wiz.definition.Definition({
+            "identifier": "A",
+            "version": "0.1.0",
+            "requirements": ["B", "C", "D"]
+        })
     )
     node = mocker.Mock(identifier="_A==0.1.0")
     requirement = Requirement("A")
@@ -1941,10 +1994,20 @@ def test_graph_update_from_requirement_multi_packages(
     mocker, mocked_resolver, mocked_package_extract, mocked_queue, options
 ):
     """Update graph from requirement."""
+    definition = wiz.definition.Definition({
+        "identifier": "A",
+        "version": "0.1.0",
+        "variants": [
+            {"identifier": "variant1"},
+            {"identifier": "variant2"},
+            {"identifier": "variant3"},
+        ]
+    })
+
     packages = [
-        wiz.package.Package(identifier="A[variant1]==0.1.0"),
-        wiz.package.Package(identifier="A[variant2]==0.1.0"),
-        wiz.package.Package(identifier="A[variant3]==0.1.0")
+        wiz.package.Package(definition, variant_index=0),
+        wiz.package.Package(definition, variant_index=1),
+        wiz.package.Package(definition, variant_index=2)
     ]
 
     nodes = [
@@ -1990,18 +2053,78 @@ def test_graph_update_from_requirement_multi_packages(
     mocked_queue.put.assert_not_called()
 
 
+def test_graph_update_from_requirements_with_invalid_requirements(
+    mocked_resolver, mocked_package_extract
+):
+    """Fail to update graph with invalid package requirements."""
+    graph = wiz.graph.Graph(mocked_resolver)
+
+    _mapping = {
+        "A==0.1.0": wiz.package.Package(
+            wiz.definition.Definition({
+                "identifier": "A",
+                "version": "0.1.0",
+                "requirements": ["!!!"],
+            })
+        ),
+    }
+
+    mocked_package_extract.side_effect = [
+        [_mapping["A==0.1.0"]],
+    ]
+
+    with pytest.raises(wiz.exception.IncorrectDefinition) as error:
+        graph.update_from_requirements([Requirement("A")], graph.ROOT)
+
+    assert (
+        "IncorrectDefinition: Package 'A==0.1.0' is incorrect "
+        "[The requirement '!!!' is incorrect]"
+    ) in str(error)
+
+
+def test_graph_update_from_requirements_with_invalid_conditions(
+    mocked_resolver, mocked_package_extract
+):
+    """Fail to update graph with invalid package conditions."""
+    graph = wiz.graph.Graph(mocked_resolver)
+
+    _mapping = {
+        "A==0.1.0": wiz.package.Package(
+            wiz.definition.Definition({
+                "identifier": "A",
+                "version": "0.1.0",
+                "conditions": ["!!!"],
+            })
+        ),
+    }
+
+    mocked_package_extract.side_effect = [
+        [_mapping["A==0.1.0"]],
+    ]
+
+    with pytest.raises(wiz.exception.IncorrectDefinition) as error:
+        graph.update_from_requirements([Requirement("A")], graph.ROOT)
+
+    assert (
+        "IncorrectDefinition: Package 'A==0.1.0' is incorrect "
+        "[The requirement '!!!' is incorrect]"
+    ) in str(error)
+
+
 def test_graph_create_node_from_package():
     """Create node in graph from package."""
-    package = wiz.package.Package({
-        "identifier": "A==0.1.0",
-        "definition": wiz.definition.Definition({"identifier": "defA"})
-    })
+    package = wiz.package.Package(
+        wiz.definition.Definition({
+            "identifier": "A",
+            "version": "0.1.0"
+        })
+    )
 
     graph = wiz.graph.Graph(None)
     graph._create_node_from_package(package)
 
-    assert graph._identifiers_per_definition == {"defA": {"A==0.1.0"}}
-    assert graph._node_mapping.keys() == ["A==0.1.0"]
+    assert graph._identifiers_per_definition == {"A": {"A==0.1.0"}}
+    assert list(graph._node_mapping.keys()) == ["A==0.1.0"]
     assert isinstance(graph._node_mapping["A==0.1.0"], wiz.graph.Node)
 
 
@@ -2069,14 +2192,17 @@ def test_graph_remove_node():
 
 def test_node():
     """Create and use node."""
-    package = wiz.package.Package({
-        "identifier": "A[V1]==0.1.0",
-        "definition": "__DEFINITION__"
+    definition = wiz.definition.Definition({
+        "identifier": "A",
+        "version": "0.1.0",
+        "variants": [{"identifier": "V1"}]
     })
 
+    package = wiz.package.Package(definition, variant_index=0)
+
     node = wiz.graph.Node(package)
     assert node.identifier == "A[V1]==0.1.0"
-    assert node.definition == "__DEFINITION__"
+    assert node.definition == definition
     assert node.package == package
     assert node.parent_identifiers == set()
 
